TIMEFORMAT='%2R'
BINDIR=".."

to=timeout
if [ "$(uname)" == "Darwin" ]; then to=gtimeout; fi # for MacOS
umask 0022

# colors
# cY=66FF33
# cN=FA895C
# cM=CCFFFF
# cT=CCFFFF
# colors from CL experiments style file
if [ -z "$cY" ]; then cY=00FF00; fi
if [ -z "$cN" ]; then cN=00BB00; fi
if [ -z "$cM" ]; then cM=FFFF00; fi
if [ -z "$cT" ]; then cT=FFBB00; fi
if [ -z "$cC" ]; then cC=FFCCFF; fi

DB="COPS" # list of directories with TRS problems
TABLE=1   # include details of individual problems
CERT=1

while getopts "d:tc" opt; do
  case "$opt" in
    d)  DB=$OPTARG;; # to replace default problem directory
    t)  TABLE=0;;
    c)  CERT=0;;
    *)  printf "illegal option $opt\n"; exit 1;;
  esac
done
shift "$((OPTIND-1))"

experiment=$1 # filename of FORT experiment script
printf "FORT EXPERIMENT $experiment\n\n"

# sanity checks
if ! [ -e $experiment.fort ]; then
  printf "file $experiments.fort is missing!\n"; exit
elif [ -d $experiment ]; then
  printf "directory $experiment already exists; "
  printf "do you want to continue [Y/N]? "
  while read -r -n 1 -s answer; do
  printf "\n\n"
  if [[ $answer = [YyNn] ]]; then
    [[ $answer = [Nn] ]] && exit
    [[ $answer = [Yy] ]] && /bin/rm -Rf $experiment; break
  fi
  exit
  done
fi

# create directory and copy TRSs into it, preserving directory structure
mkdir $experiment
for db in $DB; do
  mkdir $experiment/$db
  for TRS in $db/*.trs; do
    cp $TRS $experiment/$db
    chmod 0644 "$experiment/$TRS"
  done
done

# default timeouts
timeout=60
ctimeout=60

# default version
version=h # fort-h

# formula number
number=1

# string to control columns for table in part 2
columns="" 

printf "PART 1: run experiments\n"

# process FORT script file
while read -r line
do
  command=${line:0:2}
  if [ "$command" == "-o" ]; then
    options=${line:3}
    printf "\nFORT OPTIONS: $options\n"
  elif [ "$command" == "-t" ]; then
    timeout=${line:3}
    if ! [[ $timeout =~ ^[1-9][0-9]*$ ]]; then
      printf "\n-t SYNTAX ERROR: $timeout is not a number\n"; exit
    fi
    printf "\nTIMEOUT: $timeout second(s)\n"
  elif [ "$command" == "-T" ]; then
    ctimeout=${line:3}
    if ! [[ $ctimeout =~ ^[1-9][0-9]*$ ]]; then
      printf "\n-t SYNTAX ERROR: $ctimeout is not a number\n"; exit
    fi
    printf "\nCERTIFICATION TIMEOUT: $ctimeout second(s)\n"
  elif [ "$command" == "-v" ]; then
    version=${line:3}
    if ! [[ $version == "2.1" || $version == "h" ]]; then
      printf "\n-v ERROR: $version is not supported\n"; exit
    fi
  elif [ "$command" == "-f" ]; then
    formula=${line:3}            # input formula
    if [[ $formula =~ ^\".*\"$ ]]; then
      formula="${formula//\"/}"  # remove double quotes surrounding formula
      FORMULAS[number]=$formula  # store formula
      TIMEOUTS[number]=$timeout  # store timeout
      CTIMEOUTS[number]=$ctimeout  # store timeout for certification
      VERSIONS[number]=$version  # store version
      OPTIONS[number]=$options   # store options
      printf "\nFORMULA $number: $formula\n"
      for db in $DB; do
        for TRS in $experiment/$db/*.trs; do
          name=$(basename "$TRS" .trs) # TRS name without suffix '.trs'
          output=$experiment/$db/$name.$number.fort # output file
          certificate=$experiment/$db/$name.$number.cert
          html="-p"
          printf "output: $output\n"
          # FORT CALL with time limit; store FORT output and time
          if [[ $version == "2.1" ]]; then
           command="java -jar ./fort.jar -D"
           time ($to -k 0.1 $timeout.0 $command $options $TRS "$formula" \
            >> $output) >> $output 2>&1
          else
           command="$BINDIR/fort-h"
           time ($to -k 0.1 $timeout.0 $command -c $certificate $options \
            "$formula" $TRS >> $output) >> $output 2>&1
           answer=$(head -1 "$output")
           printf "   $answer \n"
           time ($to -k 0.1 $ctimeout $BINDIR/fortify $answer "$formula" \
            $certificate $TRS >> $output ) >> $output 2>&1
          fi
        done
      done
      ((number++))          # prepare for next formula
      columns="${columns}d" # data column for table
    else
      printf "\n-f SYNTAX ERROR: $formula\n"
      printf "%41s\n" "formula must be surrounded by double quotes"; exit
    fi
  elif [ "$command" == "-e" ]; then
      columns="${columns}e" # empty column for table
  elif [ "$command" == "-x" ]; then
    break # exit loop when encountering -x in script file
  else
    printf "\nSYNTAX ERROR: -f or -t or -T or -e or -x expected\n"; exit
  fi
done < "$experiment.fort"

cp $experiment.fort $experiment/ # copy script to results directory
chmod 0644 "$experiment/$experiment.fort"

printf "\nPART 2: generate table for web site\n"

# initialize variables for summary
number=0
for (( i=0; i<${#columns}; i++ )); do
  if [ "${columns:$i:1}" == "d" ]; then # experiment column
    ((number++))
    YES[number]=0
    CYES[number]=0
    NO[number]=0
    CNO[number]=0
    MAYBE[number]=0
    CMAYBE[number]=0
    TIMEOUT[number]=0
    CTIMEOUT[number]=0
    TOTAL[number]=0
    CTOTAL[number]=0
    TIME[number]="0.00"
    CTIME[number]="0.00"
    TIMEy[number]="0.00"
    CTIMEy[number]="0.00"
    TIMEn[number]="0.00"
    CTIMEn[number]="0.00"
  fi
done

# compute variables for summary
for db in $DB; do
  for TRS in $experiment/$db/*.trs; do
    name=$(basename "$TRS" .trs)
    number=0
    for (( i=0; i<${#columns}; i++ )); do
      if [ "${columns:$i:1}" == "d" ]; then
        ((number++))
        file=$experiment/$db/$name.$number.fort
        if [ -e $experiment/$db/$name.$number.html ]; then
          file2=$db/$name.$number.html
        else
          file2=$db/$name.$number.fort
        fi
        status=$(head -1 "$file")
        time=$(head -2 "$file" | tail -1)
        certified=$(head -3 "$file" | tail -1)
        ctime=$(head -4 "$file" | tail -1)
        if [ "$status" == "YES" ]; then
          ((YES[number]++))
          TIME[number]=`echo "${TIME[number]} $time" | \
            awk '{printf "%.2f \n", $1+$2}'`
          TIMEy[number]=`echo "${TIMEy[number]} $time" | \
            awk '{printf "%.2f \n", $1+$2}'`
          if [ "$certified" == "Certified" ]; then
            ((CYES[number]++))
            ((CTOTAL[number]++))
          else
            ((CTIMEOUT[number]++))
          fi
        elif [ "$status" == "NO" ]; then
          TIME[number]=`echo "${TIME[number]} $time" | \
            awk '{printf "%.2f \n", $1+$2}'`
          TIMEn[number]=`echo "${TIMEn[number]} $time" | \
            awk '{printf "%.2f \n", $1+$2}'`
          ((NO[number]++))
          if [ "$certified" == "Certified" ]; then
            ((CNO[number]++))
            ((CTOTAL[number]++))
          else
            ((CTIMEOUT[number]++))
          fi
        elif [ "$status" == "MAYBE" ]; then
          TIME[number]=`echo "${TIME[number]} $time" | \
            awk '{printf "%.2f \n", $1+$2}'`
          ((MAYBE[number]++))
        else
          TIME[number]=`echo "${TIME[number]} ${TIMEOUTS[number]}" | \
            awk '{printf "%.2f \n", $1+$2}'`
          ((TIMEOUT[number]++))
        fi
        ((TOTAL[number]++))
      fi
    done
  done
done

(
cat <<BLOCK
<html>
 
<head>
<meta http-equiv="content-type" content="text/html; charset=ISO-8859-1">
<link rel="stylesheet" type="text/css" 
href="http://colo1-c703.uibk.ac.at/workspace/style-wide-new.css">
<style type="text/css">
BLOCK

printf "  .no { background-color: #$cN; text-align: center }\n"
printf "  .yes { background-color: #$cY; text-align: center }\n"
printf "  .maybe { background-color: #$cM; text-align: center }\n"
printf "  .timeout { background-color: #$cT; text-align: center }\n"
printf "  .certify { background-color: #$cC; text-align: center }\n"

cat <<BLOCK
  .summary { text-align: right; padding-right: 15px }
  a:link { color: black; text-decoration: none }
</style>
<title> FORT experimental results</title>
</head>
 
<body style="background-color: #FFFFFF; text-align: left;
font-family: Helvetica, Arial, sans-serif">
 
<div class="content">
 
<h4><a href="http://cl-informatik.uibk.ac.at/software/FORT/"
target="_blank"><img
src="http://cl-informatik.uibk.ac.at/software/FORT/fort.png"
style="height:50px;"></a></h4>

<br />

<h5>FORMULAS</h5>

<table style="font-size: x-small" cellpadding="2" cellspacing="10"
border="0">
BLOCK

# COLUMN DESCRIPTIONS

number=1
number2=10112 # circled numbers, starting with 1
for (( i=0; i<${#columns}; i++ )); do
  if [ "${columns:$i:1}" == "d" ]; then
    printf "<tr><td class=\"summary\">&#$number2;</td>\n"
    printf "<td>FORT %s &nbsp; %s</td>\n" "${OPTIONS[number]}" \
      "${FORMULAS[number]}"
    # TODO pretty print formula
    ((number++))
    ((number2++))
  fi
done

cat <<BLOCK
</table>

<table style="font-size: x-small" cellpadding="2" cellspacing="3"
border="0">
BLOCK

# TABLE HEADER

printf "<tr>\n<th></th>\n"
number=10111
for (( i=0; i<${#columns}; i++ )); do
  if [ "${columns:$i:1}" == "e" ]; then
    printf "<th></th>\n"
  else
    ((number++))
    printf "<th class=\"summary\">&#$number;</th><th></th>\n"
  fi
done
printf "</tr>\n"

# SUMMARY

printf "<tr><td>version &nbsp;</td>\n"
number=1
for (( i=0; i<${#columns}; i++ )); do
  if [ "${columns:$i:1}" == "e" ]; then
    printf "<td></td>\n"
  else
    printf "<td class=\"summary\">%s</td><td class=\"summary\">&#10004;</td>\n" \
     "${VERSIONS[number]}" 
    ((number++))
  fi
done

printf "</tr>\n"
printf "<tr><td>total &nbsp;</td>\n"
number=1
for (( i=0; i<${#columns}; i++ )); do
  if [ "${columns:$i:1}" == "e" ]; then
    printf "<td></td></td>\n"
  else
    printf "<td class=\"summary\">%s</td><td class=\"summary\">%s</td>\n" \
     "${TOTAL[number]}" "${CTOTAL[number]}"
    ((number++))
  fi
done
printf "</tr>\n"

printf "<tr><td>yes &nbsp;</td>\n"
number=1
for (( i=0; i<${#columns}; i++ )); do
  if [ "${columns:$i:1}" == "e" ]; then
    printf "<td></td>\n"
  else
    printf "<td class=\"summary\" style=\"background-color: #$cY\">%s</td>\n" "${YES[number]}"
    printf "<td class=\"summary\" style=\"background-color: #$cC\">%s</td>\n" "${CYES[number]}"
    ((number++))
  fi
done

printf "<tr><td>&#8960; yes time &nbsp;</td>\n"
number=1
for (( i=0; i<${#columns}; i++ )); do
  if [ "${columns:$i:1}" == "e" ]; then
    printf "<td></td>\n"
  else
    if [ ${YES[number]} == 0 ]; then
      printf "<td></td><td></td>\n"
    else
      average=`echo "${TIMEy[number]} ${YES[number]}" | \
              awk '{printf "%.2f \n", $1/$2}'`
      printf "<td class=\"summary\">%s</td><td></td>\n" "$average"
    fi
    ((number++))
  fi
done

printf "<tr><td>total yes time &nbsp;</td>\n"
number=1
for (( i=0; i<${#columns}; i++ )); do
  if [ "${columns:$i:1}" == "e" ]; then
    printf "<td></td>\n"
  else
    printf "<td class=\"summary\">%s</td><td></td>\n" "${TIMEy[number]}"
    ((number++))
  fi
done

printf "<tr><td>no &nbsp;</td>\n"
number=1
for (( i=0; i<${#columns}; i++ )); do
  if [ "${columns:$i:1}" == "e" ]; then
    printf "<td></td>\n"
  else
    printf "<td class=\"summary\" style=\"background-color: #$cN\">%s</td>\n" "${NO[number]}"
    printf "<td class=\"summary\" style=\"background-color: #$cC\">%s</td>\n" "${CNO[number]}"
    ((number++))
  fi
done

printf "<tr><td>&#8960; no time &nbsp;</td>\n"
number=1
for (( i=0; i<${#columns}; i++ )); do
  if [ "${columns:$i:1}" == "e" ]; then
    printf "<td></td>\n"
  else
    if [ ${NO[number]} == 0 ]; then
      printf "<td></td><td></td>\n"
    else
      average=`echo "${TIMEn[number]} ${NO[number]}" | \
              awk '{printf "%.2f \n", $1/$2}'`
      printf "<td class=\"summary\">%s</td><td></td>\n" "$average"
    fi
    ((number++))
  fi
done

printf "<tr><td>total no time &nbsp;</td>\n"
number=1
for (( i=0; i<${#columns}; i++ )); do
  if [ "${columns:$i:1}" == "e" ]; then
    printf "<td></td>\n"
  else
    printf "<td class=\"summary\">%s</td><td></td>\n" "${TIMEn[number]}"
    ((number++))
  fi
done

printf "<tr><td>maybe &nbsp;</td>\n"
number=1
for (( i=0; i<${#columns}; i++ )); do
  if [ "${columns:$i:1}" == "e" ]; then
    printf "<td></td>\n"
  else
    printf "<td class=\"summary\" style=\"background-color: #$cM\">%s</td><td></td>\n" "${MAYBE[number]}"
    ((number++))
  fi
done

printf "<tr><td>timeout &nbsp;</td>\n"
number=1
for (( i=0; i<${#columns}; i++ )); do
  if [ "${columns:$i:1}" == "e" ]; then
    printf "<td></td>\n"
  else
    printf "<td class=\"summary\" width="50px" style=\"background-color: "
    printf "$cT\">(${TIMEOUTS[number]}"
    printf "s) %s</td>\n" "${TIMEOUT[number]}"
    printf "<td class=\"summary\" width="50px" style=\"background-color: "
    printf "$cT\">(${CTIMEOUTS[number]}"
    printf "s) %s</td>\n" "${CTIMEOUT[number]}"
    ((number++))
  fi
done

printf "<tr><td>&#8960; time &nbsp;</td>\n"
number=1
for (( i=0; i<${#columns}; i++ )); do
  if [ "${columns:$i:1}" == "e" ]; then
    printf "<td></td>\n"
  else
    if [ ${TOTAL[number]} == 0 ]; then
      printf "<td></td><td></td>\n"
    else
      average=`echo "${TIME[number]} ${TOTAL[number]}" | \
              awk '{printf "%.2f \n", $1/$2}'`
      printf "<td class=\"summary\">%s</td><td></td>\n" "$average"
    fi
    ((number++))
  fi
done

printf "<tr><td>total time &nbsp;</td>\n"
number=1
for (( i=0; i<${#columns}; i++ )); do
  if [ "${columns:$i:1}" == "e" ]; then
    printf "<td></td>\n"
  else
    printf "<td class=\"summary\">%s</td><td></td>\n" "${TIME[number]}"
    ((number++))
  fi
done

if [ $TABLE == 1 ]; then # generate detailed results
printf "<tr><td>&nbsp;</td></tr>\n"
# TABLE ROWS
for db in $DB; do
  for TRS in $experiment/$db/*.trs; do
    name=$(basename "$TRS" .trs)
    printf "<tr>\n<td><a href=\"$db/$name.trs\">$db/$name</a>"
    printf " &nbsp;</td>\n"
    number=0
    for (( i=0; i<${#columns}; i++ )); do
      printf "<td style=\"width:"
      if [ "${columns:$i:1}" == "e" ]; then
        printf "25px\"></td>\n"
      else
        ((number++))
        file=$experiment/$db/$name.$number.fort
        if [ -e $experiment/$db/$name.$number.html ]; then
          file2=$db/$name.$number.html
        else
          file2=$db/$name.$number.fort
        fi
        printf "50px\" class=\""
        status=$(head -1 "$file")
        time=$(head -2 "$file" | tail -1)
        if [ "$status" == "YES" ]; then
          printf "yes\">"
          printf "<a href=\"$file2\">$time</a></td>\n"
        elif [ "$status" == "NO" ]; then
          printf "no\">"
          printf "<a href=\"$file2\">$time</a></td>\n"
        elif [ "$status" == "MAYBE" ]; then
          printf "maybe\">"
          printf "<a href=\"$file2\">$time</a></td>\n"
        else
          printf "timeout\">&infin;</td>\n"
        fi
        if [[ "$status" == "YES" || "$status" == "NO" ]]; then
          certified=$(head -3 "$file" | tail -1)
          ctime=$(head -4 "$file" | tail -1)
          if [[ "$certified" == "Certified" ]]; then
            printf "<td style=\"width:50px\" class=\"certify\">"
            printf "$ctime</td>\n"
          else
            printf "<td></td>"
          fi
        else
          printf "<td></td>"
        fi
      fi
    done
    printf "</tr>\n"
  done
  printf "<tr><td>&nbsp;</td></tr>\n"
done
fi

# FINISH

printf "</table>\n\n</div>\n\n</body>\n\n</html>\n"
) > $experiment/index.html

