-- | Decision procedure used internally by the synthesis tool
module Satisfiability (
  satisfies,
  satisfiesManySort
) where

import qualified Data.Rewriting.Rule as R

import qualified Formula as Fm
import qualified Formula.Automata as Fm
import qualified Formula.Simplify as Fm
import qualified RRn
import qualified TA
import Util.Signature

-- | @satisfies sig trss f@ checks if the signature @sig@ and
-- TRSs @trss@ satisfy the formula @f@
satisfies :: (Ord f, Ord v) => [(f, Int)] -> [[R.Rule f v]] -> Fm.Formula -> Bool
satisfies sig = satisfiesManySort sig'
  where sig' = [(f, replicate i (), ()) | (f, i) <- sig]

-- | @satisfies sig trss f@ checks if the many-sorted signature @sig@ and
-- TRSs @trss@ satisfy the formula @f@
satisfiesManySort :: (Ord f, Ord v, Ord s)
  => Signature s f -> [[R.Rule f v]] -> Fm.Formula -> Bool
satisfiesManySort sig trss form = go False form'
  where
    form' = Fm.exForm . Fm.unrestrict $ form
    trss' = [[R.map Right id r | r <- trs] | trs <- trss]
    sig'  = [(Right f, ss, s) | (f,ss,s) <- sig]
    freshConsts = [ [(Left (s, i), [], s) | i <- [0..] :: [Int]]
                  | s <- sorts sig'
                  ]

    go :: Bool -> Fm.Formula -> Bool
    go b (Fm.FNot f) = go (not b) f
    go b (Fm.FExists f) = go b f
    go b f = if b then res else not res
      where
        ((_, rrn, _), _) = Fm.evalM $ Fm.mkFormula False sig' freshConsts trss' f
        res = TA.isEmpty (RRn.ta rrn)
